searchNodes=[{"ref":"Uploader.html","title":"Uploader","type":"behaviour","doc":""},{"ref":"Uploader.html#c:maybe_copy_file/3","title":"Uploader.maybe_copy_file/3","type":"callback","doc":""},{"ref":"Uploader.html#store_files/1","title":"Uploader.store_files/1","type":"function","doc":"Store all the uploaded files. This function is expected to be called inside a transaction: Multi.new() |&gt; Multi.insert(:user, user_changeset) |&gt; Multi.run(:upload_files, fn _repo, %{user: user} -&gt; Uploader.store_files(user) end) |&gt; Repo.transaction() |&gt; case do {:ok, %{user: user}} -&gt; {:ok, user} {:error, :user, %Ecto.Changeset{} = changeset, _changes} -&gt; {:error, changeset} {:error, :upload_files, {:file_path_exists, file_path}, _changes} -&gt; raise &quot;file upload failed: path \\&quot;\#{file_path}\\&quot; already exists&quot; end This function fetches all the uploadable schema fields (with all of their options) from the given Schema struct (see Uploader.Ecto.UploadableFields), and for each field calls the maybe_copy_file/4 callback function. This callback function is called from the module defined in the :uploader option of the field (by default, the module Uploader.LocalUploader is used). Below is an example on how to define a custom module implementing the maybe_copy_file/4 callback function in the Schema: schema &quot;users&quot; do uploadable_field :avatar_image, uploader: AmazonS3Uploader # other fields end In case of success {:ok, nil} is returned, otherwise {:error, error_tuple} is returned with error_tuple being a two-elements tuple describing the error. This tuple is in fact the returned error value of the call to the maybe_copy_file/4 function. An example of a returned error value is {:error, {:file_path_exists, file_path}}."},{"ref":"Uploader.Ecto.Changeset.html","title":"Uploader.Ecto.Changeset","type":"module","doc":""},{"ref":"Uploader.Ecto.Changeset.html#cast_with_upload/3","title":"Uploader.Ecto.Changeset.cast_with_upload/3","type":"function","doc":"Returns a changeset for the given struct (see Ecto.Changeset.cast/4). Casts file uploads (Plug.Upload structs) into a generated UUID filename or, if the :cast option is present, into a value obtained by a call to the user-defined cast function. In order not to lose the Plug.Upload structs after casting, these structs are stored into virtual fields named by the original fields that held the file uploads prefixed by &quot;uploaded_&quot;. Example: A param named &quot;image&quot; contains a file upload (Plug.Upload struct); cast_with_upload/3 casts the Plug.Upload struct into a filename; the changeset field :image contains the filename; the changeset field :uploaded_image contains the Plug.Upload struct."},{"ref":"Uploader.Ecto.UploadableFields.html","title":"Uploader.Ecto.UploadableFields","type":"behaviour","doc":"Provides a macro for defining uploadable fields. When calling uploadable_field/2, two fields are created: a virtual field is created named by the given name prefixed by &quot;uploaded_&quot; and holds the Plug.Upload struct (representing the uploaded file). a field with the given name is created holding the casted Plug.Upload struct (typically the struct is casted into a filename). The code below uploadable_field :image will generate field :image, :string field :uploaded_image, :any, virtual: true Using this module (use) provides the caller module with the function get_uploadable_fields\\0 listing all the uploadable fields and their options. The following options may be given to uploadable fields: :cast: a function that casts a Plug.Upload struct into the value to be stored in the database. directory: the base directory containing the file uploads. filename: a function that generates the filename based on the given struct. on_file_exists: specifies the strategy to apply if the file path already exists. Its value may be: :overwrite: overwrite the file if the file path already exists :compare_hash: do not copy the file if the file path already exists; if the hashes of the files&#39; content are not equal, return an error. print: a function that prints the field (typically used be the view). type: the field type."},{"ref":"Uploader.Ecto.UploadableFields.html#c:get_uploadable_fields/0","title":"Uploader.Ecto.UploadableFields.get_uploadable_fields/0","type":"callback","doc":""},{"ref":"Uploader.Ecto.UploadableFields.html#uploadable_field/2","title":"Uploader.Ecto.UploadableFields.uploadable_field/2","type":"macro","doc":""},{"ref":"Uploader.FileHasher.html","title":"Uploader.FileHasher","type":"module","doc":""},{"ref":"Uploader.FileHasher.html#hash_file_content/1","title":"Uploader.FileHasher.hash_file_content/1","type":"function","doc":"Compute and return the hash of a file."},{"ref":"Uploader.HTML.UploadHelpers.html","title":"Uploader.HTML.UploadHelpers","type":"module","doc":"Provides a view helper to render the URL to uploaded files."},{"ref":"Uploader.HTML.UploadHelpers.html#upload_url/3","title":"Uploader.HTML.UploadHelpers.upload_url/3","type":"function","doc":"Renders the URL to an uploaded file. You may specify the :base_url option to prepend the base URL."},{"ref":"Uploader.LocalUploader.html","title":"Uploader.LocalUploader","type":"module","doc":"Store file uploads locally. This module implements the maybe_copy_file/4 callback function defined in the Uploader module."},{"ref":"Uploader.LocalUploader.html#maybe_copy_file/3","title":"Uploader.LocalUploader.maybe_copy_file/3","type":"function","doc":"Copies the uploaded file to a given new file path. The third optional argument on_file_exists allows to specify what should happen in case the filename already exists. If no value is given and the filename already exists, the file is not copied and an error is returned. This behaviour may be changed by setting the value to: :overwrite: overwrite the file if the file path already exists. :compare_hash: do not copy the file if the file path already exists; if the hashes of the files&#39; content are not equal, return an error. This function returns :ok in case of success. If an error occurs, a two-elements tuple is returned with the first element being :error and the second element another two-elements tuple describing the type of error and a value. For example: {:error, {:file_path_exists, file_path}}."},{"ref":"readme.html","title":"Uploader","type":"extras","doc":"Uploader Uploader helps you with storing file uploads. There are numerous file upload libraries for Elixir already available, however this library was made in order to have a finer control over how Plug.Upload are casted into schema fields, over the filenames that should be generated, the strategy to apply for existing file paths, etc. Imagine a single image upload that must to be stored multiple times with filenames in different languages for SEO purposes. The writing of this library was motivated to address such advanced use cases."},{"ref":"readme.html#define-the-schema-fields-holding-uploads","title":"Uploader - Define the Schema fields holding uploads","type":"extras","doc":"defmodule MyApp.User do use Ecto.Schema use Uploader.Ecto.UploadableFields schema &quot;posts&quot; do uploadable_field :avatar_image end end You may import :uploader&#39;s formatter configuration by importing uploader into your .formatter.exs file (this allows for example to keep uploadable_field :avatar_image without parentheses when running mix format). [ import_deps: [:ecto, :phoenix, :uploader], #... ]"},{"ref":"readme.html#cast-plug-upload-structs","title":"Uploader - Cast Plug.Upload structs","type":"extras","doc":"In order to cast file uploads (Plug.Upload structs) into filenames, you must call cast_with_upload/3 from the Uploader.Ecto.Changeset module for the given Schema struct and params. Note that, in addition to casting file uploads, cast_with_upload/3 implicitly calls Ecto.Changeset.cast/4 for the given params and returns the changeset. import Uploader.Ecto.Changeset schema &quot;user&quot; do uploadable_field :avatar_image end @required_fields ~w(avatar_image)a @optional_fields ~w()a def changeset(user, attrs) do user |&gt; cast_with_upload(attrs, @required_fields ++ @optional_fields) |&gt; validate_required(@required_fields) end"},{"ref":"readme.html#copy-the-uploaded-files","title":"Uploader - Copy the uploaded files","type":"extras","doc":"This library expects the storing of files to happen in a transaction. This is done with a call to Uploader.store_files/1 given a Schema struct. Multi.new() |&gt; Multi.insert(:user, user_changeset) |&gt; Multi.run(:upload_files, fn _repo, %{user: user} -&gt; Uploader.store_files(user) end) |&gt; Repo.transaction() |&gt; case do {:ok, %{user: user}} -&gt; {:ok, user} {:error, :user, %Ecto.Changeset{} = changeset, _changes} -&gt; {:error, changeset} {:error, :upload_files, {:file_path_exists, file_path}, _changes} -&gt; raise &quot;file upload failed: path \\&quot;\#{file_path}\\&quot; already exists&quot; end"},{"ref":"readme.html#print-urls-to-uploaded-files","title":"Uploader - Print URLs to uploaded files","type":"extras","doc":"A view helper is provided in order to print the URL of uploaded files. Open up the entrypoint defining your web interface, such as MyAppWeb, and add the line below into the view function&#39;s quote block. def view do quote do # some code import Uploader.HTML.UploadHelpers end end The helper below prints the URL for an uploaded file: &lt;%= upload_url(user, :avatar_image) %&gt; You may also prepend the URL with a base URL: &lt;%= upload_url(user, :avatar_image, base_url: upload_path()) %&gt;"},{"ref":"readme.html#field-options","title":"Uploader - Field options","type":"extras","doc":"The uploadable_field/2 macro may optionally receive options to alter the way Plug.Upload structs are casted, how filenames are generated, etc. Here is a list of options: :cast: a function that casts a Plug.Upload struct into the value to be stored in the database. directory: the base directory containing the file uploads. filename: a function that generates the filename based on the given struct. on_file_exists: specifies the strategy to apply if the file path already exists. Its value may be: :overwrite: overwrite the file if the file path already exists :compare_hash: do not copy the file if the file path already exists; if the hashes of the files&#39; content are not equal, return an error. print: a function that prints the field (typically used be the view). type: the field type. Example case for custom options A CMS allows the upload of a blog post&#39;s cover image. The blog post is available in multiple languages and the cover image must be stored multiple times in different languages for SEO purposes. For example, the cover image for my post about my favorite books must have the name &quot;my-favorite-books.jpg&quot; in English, &quot;mes-livres-preferes.jpg&quot; in French, etc. We should then store all the images&#39; filenames and retrieve the right filename according to the language used by the reader. The image upload (Plug.Upload) must be casted into a map of filenames in different languages: %{ &quot;en&quot; =&gt; &quot;my-favorite-books.jpg&quot;, &quot;fr&quot; =&gt; &quot;mes-livres-preferes.jpg&quot;, &quot;nl&quot; =&gt; &quot;mijn-lievelingsboeken.jpg&quot; } We want those filenames to have the same name as the post&#39;s URL slug (which are held by another Schema field :slug) with the file extension appended. I.e. if the English URL slug for that post is my-favorite-books, the cover image will be named my-favorite-books.jpg in English. Below are the Schema fields with the custom options to make that work. The code below uses the translatable_field/1 macro from the i18n_helpers package. A translatable field holds a map of values for different languages as the map shown above, and generates a virtual field (field prepended by &quot;translated_&quot;) holding the translated field. For example the :slug field may contain a map such as %{&quot;en&quot; =&gt; &quot;favorite-books&quot;, &quot;fr&quot; =&gt; &quot;livres-preferes&quot;} and :translated_slug will contain livres-preferes if the struct has been translated in French. See i18n_helpers&#39;s readme. translatable_field :slug translatable_field :cover_image uploadable_field :cover_image, cast: &amp;User.cast_cover_image/2, directory: @uploads_directory, filename: &amp;User.filename_cover_image/2, on_file_exists: :compare_hash, print: &amp;User.print_cover_image/2, type: :map # Cast the Plug.Upload struct into a map of filenames per language. def cast_cover_image( %Plug.Upload{filename: uploaded_filename}, %Ecto.Changeset{} = changeset ) do slugs = fetch_field!(changeset, :slug) extension = Path.extname(uploaded_filename) |&gt; String.downcase() Enum.map(slugs, fn {language, slug} -&gt; {language, slug &lt;&gt; extension} end) |&gt; Enum.into(%{}) end # Return the list of filenames to be stored. def filename_cover_image(%User{cover_image: cover_image}, _field_name) do Map.values(cover_image) end # Return the filename to be printed. def print_cover_image(%User{translated_cover_image: translated_cover_image}, _field_name) do translated_cover_image end"},{"ref":"readme.html#installation","title":"Uploader - Installation","type":"extras","doc":"Add uploader for Elixir as a dependency in your mix.exs file: def deps do [ {:uploader, &quot;~&gt; 0.1.0&quot;} ] end"},{"ref":"readme.html#hexdocs","title":"Uploader - HexDocs","type":"extras","doc":"HexDocs documentation can be found at https://hexdocs.pm/uploader."}]